from django.forms import ModelForm, DateInput
from opcalendar.models import Event, EventMember, EventCategory
from django import forms


class EventForm(ModelForm):
    class Meta:
        model = Event
        # datetime-local is a HTML5 input type, format to make date time show on fields
        widgets = {
            "start_time": DateInput(
                attrs={"type": "datetime-local"}, format="%Y-%m-%dT%H:%M"
            ),
            "end_time": DateInput(
                attrs={"type": "datetime-local"}, format="%Y-%m-%dT%H:%M"
            ),
        }
        exclude = ["user", "eve_character", "created_date"]

    def __init__(self, *args, **kwargs):
        super(EventForm, self).__init__(*args, **kwargs)
        # input_formats to parse HTML5 datetime-local input to datetime field
        self.fields["start_time"].input_formats = ("%Y-%m-%dT%H:%M",)
        self.fields["end_time"].input_formats = ("%Y-%m-%dT%H:%M",)


class SignupForm(forms.Form):
    username = forms.CharField(
        widget=forms.TextInput(
            attrs={"class": "form-control", "placeholder": "Username"}
        )
    )
    password = forms.CharField(
        widget=forms.PasswordInput(
            attrs={"class": "form-control", "placeholder": "Password"}
        )
    )


class AddMemberForm(forms.ModelForm):
    class Meta:
        model = EventMember
        fields = ["user"]


class AddCategoryForm(forms.ModelForm):
    class Meta:
        model = EventCategory
        fields = "__all__"



from django.urls import path

from . import views

app_name = "opcalendar"
urlpatterns = [
    path("index", views.index, name="index"),
    path("", views.CalendarView.as_view(), name="calendar"),
    path("event/new/", views.create_event, name="event_new"),
    path("add_ingame_calendar/", views.add_ingame_calendar, name="add_ingame_calendar"),
    path("event/edit/<int:pk>/", views.EventEdit.as_view(), name="event_edit"),
    path("event/<int:event_id>/details/", views.event_details, name="event-detail"),
    path(
        "ingame/event/<int:event_id>/details/",
        views.ingame_event_details,
        name="ingame-event-detail",
    ),
    path(
        "add_eventmember/<int:event_id>", views.add_eventmember, name="add_eventmember"
    ),
    path("event/<int:event_id>/remove", views.EventDeleteView, name="remove_event"),
]



from django.contrib import admin
from opcalendar.models import (
    Event,
    EventCategory,
    EventSignal,
    WebHook,
    EventHost,
    EventImport,
    Owner,
    IngameEvents,
)


class WebHookAdmin(admin.ModelAdmin):
    list_display = ("name", "enabled")


admin.site.register(WebHook, WebHookAdmin)


class EventSignalAdmin(admin.ModelAdmin):
    list_display = ("get_webhook", "ignore_past_fleets")

    def get_webhook(self, obj):
        return obj.webhook.name

    get_webhook.short_description = "Webhook Name"
    get_webhook.admin_order_field = "webhook__name"


admin.site.register(EventSignal, EventSignalAdmin)


class EventCategoryAdmin(admin.ModelAdmin):
    model = EventCategory


admin.site.register(EventCategory, EventCategoryAdmin)


class EventHostAdmin(admin.ModelAdmin):
    model = EventHost


admin.site.register(EventHost, EventHostAdmin)


class EventHostAdmin(admin.ModelAdmin):
    model = EventImport


admin.site.register(EventImport, EventHostAdmin)


admin.site.register(Owner)

admin.site.register(IngameEvents)

admin.site.register(Event)



from .app_settings import (
    OPCALENDAR_NOTIFY_IMPORTS,
)

from django.dispatch import receiver


from django.db.models.signals import post_save, pre_delete
from .models import Event, EventSignal, IngameEvents
import datetime
from django.utils import timezone

from esi.clients import EsiClientProvider

from .app_settings import get_site_url

from allianceauth.services.hooks import get_extension_logger

logger = get_extension_logger(__name__)

RED = 16711710
BLUE = 42751
GREEN = 6684416

esi = EsiClientProvider()


@receiver(post_save, sender=Event)
@receiver(post_save, sender=IngameEvents)
def fleet_saved(sender, instance, created, **kwargs):

    # Translate titles if we have ingame events
    if sender == IngameEvents:
        instance.visibility = ""

    if "import" not in instance.visibility or OPCALENDAR_NOTIFY_IMPORTS:
        try:
            logger.debug("New signal fleet created for %s" % instance.title)

            # Translate titles if we have ingame events
            if sender == IngameEvents:

                url = get_site_url() + "/opcalendar/ingame/event/{}/details/".format(
                    instance.pk
                )

                message = "New Event Created From API Feed"

                # Get the entity name from owner name
                entity_id = esi.client.Search.get_search(
                    categories=[instance.owner_type],
                    search=instance.owner_name,
                    strict=True,
                ).results()[instance.owner_type][0]

                logger.debug("Entity data is %s" % entity_id)

                main_char = instance.owner.character.character.character_name

                formup_system = instance.owner_name

                title = instance.title

                eve_time = instance.event_start_date

                fc = instance.owner_name

                # Setup portrait URL based on owner type
                if instance.owner_type == "alliance":
                    portrait = "https://images.evetech.net/alliances/{}/logo".format(
                        entity_id
                    )
                    ticker = "[{}]".format(
                        esi.client.Alliance.get_alliances_alliance_id(
                            alliance_id=entity_id
                        ).results()["ticker"]
                    )

                if instance.owner_type == "corporation":
                    portrait = "https://images.evetech.net/corporations/{}/logo".format(
                        entity_id
                    )
                    ticker = "[{}]".format(
                        esi.client.Corporation.get_corporations_corporation_id(
                            corporation_id=entity_id
                        ).results()["ticker"]
                    )

                if instance.owner_type == "character":
                    portrait = (
                        "https://images.evetech.net/characters/{}/portrait".format(
                            entity_id
                        )
                    )
                    ticker = ""

                logger.debug("Portrait url is %s" % portrait)

                character_name = instance.owner_name

            else:

                url = get_site_url() + "/opcalendar/event/{}/details/".format(
                    instance.pk
                )

                if "import" in instance.visibility:
                    message = "New Public NPSI Fleet Event From API"
                else:
                    message = "New Fleet Event"

                main_char = instance.eve_character

                formup_system = instance.formup_system

                title = instance.title

                eve_time = instance.start_time

                fc = instance.fc

                portrait = main_char.portrait_url_64

                character_name = main_char.character_name

                ticker = "{}".format(main_char.corporation_ticker)

            # If we update instead of delete
            if not created:
                message = "Event Updated"
                col = BLUE
            else:
                col = GREEN

            embed = {
                "title": message,
                "description": ("**{}** from **{}**".format(title, formup_system)),
                "url": url,
                "color": col,
                "fields": [
                    {"name": "FC", "value": fc, "inline": True},
                    {
                        "name": "Eve Time",
                        "value": eve_time.strftime("%Y-%m-%d %H:%M:%S"),
                    },
                ],
                "footer": {
                    "icon_url": portrait,
                    "text": "{}  {}".format(character_name, ticker),
                },
            }

            hooks = EventSignal.objects.all().select_related("webhook")
            old = datetime.datetime.now(timezone.utc) > eve_time

            for hook in hooks:
                if hook.webhook.enabled:
                    if old and hook.ignore_past_fleets:
                        continue
                    hook.webhook.send_embed(embed)

        except Exception as e:
            logger.error(e)
            pass  # shits fucked... Don't worry about it...


@receiver(pre_delete, sender=Event)
@receiver(pre_delete, sender=IngameEvents)
def fleet_deleted(sender, instance, **kwargs):

    # Translate titles if we have ingame events
    if sender == IngameEvents:
        instance.visibility = ""

    if "import" not in instance.visibility or OPCALENDAR_NOTIFY_IMPORTS:
        try:
            logger.debug("New signal fleet deleted for %s" % instance.title)
            url = get_site_url() + "/optimer/"

            # Translate titles if we have ingame events
            if sender == IngameEvents:

                message = "Event Removed Via API Feed"

                # Get the entity name from owner name
                entity_id = esi.client.Search.get_search(
                    categories=[instance.owner_type],
                    search=instance.owner_name,
                    strict=True,
                ).results()[instance.owner_type][0]

                logger.debug("Entity data is %s" % entity_id)

                main_char = instance.owner.character.character.character_name

                formup_system = instance.owner_name

                title = instance.title

                eve_time = instance.event_start_date

                fc = instance.owner_name

                # Setup portrait URL based on owner type
                if instance.owner_type == "alliance":
                    portrait = "https://images.evetech.net/alliances/{}/logo".format(
                        entity_id
                    )
                    ticker = "[{}]".format(
                        esi.client.Alliance.get_alliances_alliance_id(
                            alliance_id=entity_id
                        ).results()["ticker"]
                    )

                if instance.owner_type == "corporation":
                    portrait = "https://images.evetech.net/corporations/{}/logo".format(
                        entity_id
                    )
                    ticker = "[{}]".format(
                        esi.client.Corporation.get_corporations_corporation_id(
                            corporation_id=entity_id
                        ).results()["ticker"]
                    )

                if instance.owner_type == "character":
                    portrait = (
                        "https://images.evetech.net/characters/{}/portrait".format(
                            entity_id
                        )
                    )
                    ticker = ""

                logger.debug("Portrait url is %s" % portrait)

                character_name = instance.owner_name

            else:

                message = "Fleet Event Removed"

                main_char = instance.eve_character

                formup_system = instance.formup_system

                title = instance.title

                eve_time = instance.start_time

                fc = instance.fc

                portrait = main_char.portrait_url_64

                character_name = main_char.character_name

                ticker = "{}".format(main_char.corporation_ticker)

            embed = {
                "title": message,
                "description": (
                    "**{}** from **{}** has been cancelled".format(title, formup_system)
                ),
                "url": url,
                "color": RED,
                "fields": [
                    {"name": "FC", "value": fc, "inline": True},
                    {
                        "name": "Eve Time",
                        "value": eve_time.strftime("%Y-%m-%d %H:%M:%S"),
                    },
                ],
                "footer": {
                    "icon_url": portrait,
                    "text": "{}  {}".format(character_name, ticker),
                },
            }

            hooks = EventSignal.objects.all().select_related("webhook")
            old = datetime.datetime.now(timezone.utc) > eve_time

            for hook in hooks:
                if hook.webhook.enabled:
                    if old and hook.ignore_past_fleets:
                        continue
                    hook.webhook.send_embed(embed)

        except Exception as e:
            logger.error(e)
            pass  # shits fucked... Don't worry about it...
